@@ -5,21 +5,23 @@ $drivers["sqlite2"] = "SQLite 2";
 if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 	$possible_drivers = array((isset($_GET["sqlite"]) ? "SQLite3" : "SQLite"), "PDO_SQLite");
 	define("DRIVER", (isset($_GET["sqlite"]) ? "sqlite" : "sqlite2"));
-	if (extension_loaded(isset($_GET["sqlite"]) ? "sqlite3" : "sqlite")) {
+	if (class_exists(isset($_GET["sqlite"]) ? "SQLite3" : "SQLiteDatabase")) {
 		if (isset($_GET["sqlite"])) {
-			
+
 			class Min_SQLite {
-				var $extension = "SQLite3", $server_info, $affected_rows, $error, $_link;
-				
+				var $extension = "SQLite3", $server_info, $affected_rows, $errno, $error, $_link;
+
 				function Min_SQLite($filename) {
 					$this->_link = new SQLite3($filename);
 					$version = $this->_link->version();
 					$this->server_info = $version["versionString"];
 				}
-				
+
 				function query($query) {
 					$result = @$this->_link->query($query);
+					$this->error = "";
 					if (!$result) {
+						$this->errno = $this->_link->lastErrorCode();
 						$this->error = $this->_link->lastErrorMsg();
 						return false;
 					} elseif ($result->numColumns()) {
@@ -28,15 +30,18 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 					$this->affected_rows = $this->_link->changes();
 					return true;
 				}
-				
+
 				function quote($string) {
-					return "'" . $this->_link->escapeString($string) . "'";
+					return (is_utf8($string)
+						? "'" . $this->_link->escapeString($string) . "'"
+						: "x'" . reset(unpack('H*', $string)) . "'"
+					);
 				}
-				
+
 				function store_result() {
 					return $this->_result;
 				}
-				
+
 				function result($query, $field = 0) {
 					$result = $this->query($query);
 					if (!is_object($result)) {
@@ -46,22 +51,22 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 					return $row[$field];
 				}
 			}
-			
+
 			class Min_Result {
 				var $_result, $_offset = 0, $num_rows;
-				
+
 				function Min_Result($result) {
 					$this->_result = $result;
 				}
-				
+
 				function fetch_assoc() {
 					return $this->_result->fetchArray(SQLITE3_ASSOC);
 				}
-				
+
 				function fetch_row() {
 					return $this->_result->fetchArray(SQLITE3_NUM);
 				}
-				
+
 				function fetch_field() {
 					$column = $this->_offset++;
 					$type = $this->_result->columnType($column);
@@ -71,25 +76,26 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 						"charsetnr" => ($type == SQLITE3_BLOB ? 63 : 0), // 63 - binary
 					);
 				}
-				
+
 				function __desctruct() {
 					return $this->_result->finalize();
 				}
 			}
-			
+
 		} else {
-			
+
 			class Min_SQLite {
 				var $extension = "SQLite", $server_info, $affected_rows, $error, $_link;
-				
+
 				function Min_SQLite($filename) {
 					$this->server_info = sqlite_libversion();
 					$this->_link = new SQLiteDatabase($filename);
 				}
-				
+
 				function query($query, $unbuffered = false) {
 					$method = ($unbuffered ? "unbufferedQuery" : "query");
 					$result = @$this->_link->$method($query, SQLITE_BOTH, $error);
+					$this->error = "";
 					if (!$result) {
 						$this->error = $error;
 						return false;
@@ -99,15 +105,15 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 					}
 					return new Min_Result($result);
 				}
-				
+
 				function quote($string) {
 					return "'" . sqlite_escape_string($string) . "'";
 				}
-				
+
 				function store_result() {
 					return $this->_result;
 				}
-				
+
 				function result($query, $field = 0) {
 					$result = $this->query($query);
 					if (!is_object($result)) {
@@ -117,17 +123,17 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 					return $row[$field];
 				}
 			}
-			
+
 			class Min_Result {
 				var $_result, $_offset = 0, $num_rows;
-				
+
 				function Min_Result($result) {
 					$this->_result = $result;
 					if (method_exists($result, 'numRows')) { // not available in unbuffered query
 						$this->num_rows = $result->numRows();
 					}
 				}
-				
+
 				function fetch_assoc() {
 					$row = $this->_result->fetch(SQLITE_ASSOC);
 					if (!$row) {
@@ -139,11 +145,11 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 					}
 					return $return;
 				}
-				
+
 				function fetch_row() {
 					return $this->_result->fetch(SQLITE_NUM);
 				}
-				
+
 				function fetch_field() {
 					$name = $this->_result->fieldName($this->_offset++);
 					$pattern = '(\\[.*]|"(?:[^"]|"")*"|(.+))';
@@ -157,47 +163,63 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 						"orgtable" => $table,
 					);
 				}
-				
+
 			}
-			
+
 		}
-		
+
 	} elseif (extension_loaded("pdo_sqlite")) {
 		class Min_SQLite extends Min_PDO {
 			var $extension = "PDO_SQLite";
-			
+
 			function Min_SQLite($filename) {
 				$this->dsn(DRIVER . ":$filename", "", "");
 			}
 		}
-		
+
 	}
 
 	if (class_exists("Min_SQLite")) {
 		class Min_DB extends Min_SQLite {
-			
+
 			function Min_DB() {
 				$this->Min_SQLite(":memory:");
 			}
-			
+
 			function select_db($filename) {
-				if (is_readable($filename) && $this->query("ATTACH " . $this->quote(ereg("(^[/\\]|:)", $filename) ? $filename : dirname($_SERVER["SCRIPT_FILENAME"]) . "/$filename") . " AS a")) { // is_readable - SQLite 3
+				if (is_readable($filename) && $this->query("ATTACH " . $this->quote(preg_match("~(^[/\\\\]|:)~", $filename) ? $filename : dirname($_SERVER["SCRIPT_FILENAME"]) . "/$filename") . " AS a")) { // is_readable - SQLite 3
 					$this->Min_SQLite($filename);
 					return true;
 				}
 				return false;
 			}
-			
+
 			function multi_query($query) {
 				return $this->_result = $this->query($query);
 			}
-			
+
 			function next_result() {
 				return false;
 			}
 		}
 	}
-	
+
+
+
+	class Min_Driver extends Min_SQL {
+
+		function insertUpdate($table, $rows, $primary) {
+			$values = array();
+			foreach ($rows as $set) {
+				$values[] = "(" . implode(", ", $set) . ")";
+			}
+			return queries("REPLACE INTO " . table($table) . " (" . implode(", ", array_keys(reset($rows))) . ") VALUES\n" . implode(",\n", $values));
+		}
+
+	}
+
+
+
 	function idf_escape($idf) {
 		return '"' . str_replace('"', '""', $idf) . '"';
 	}
@@ -215,7 +237,7 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 	}
 
 	function limit($query, $where, $limit, $offset = 0, $separator = " ") {
-		return " $query$where" . (isset($limit) ? $separator . "LIMIT $limit" . ($offset ? " OFFSET $offset" : "") : "");
+		return " $query$where" . ($limit !== null ? $separator . "LIMIT $limit" . ($offset ? " OFFSET $offset" : "") : "");
 	}
 
 	function limit1($query, $where) {
@@ -245,9 +267,12 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 	}
 
 	function table_status($name = "") {
+		global $connection;
 		$return = array();
-		foreach (get_rows("SELECT name AS Name, type AS Engine FROM sqlite_master WHERE type IN ('table', 'view')" . ($name != "" ? " AND name = " . q($name) : "")) as $row) {
+		foreach (get_rows("SELECT name AS Name, type AS Engine FROM sqlite_master WHERE type IN ('table', 'view') " . ($name != "" ? "AND name = " . q($name) : "ORDER BY name")) as $row) {
+			$row["Oid"] = 1;
 			$row["Auto_increment"] = "";
+			$row["Rows"] = $connection->result("SELECT COUNT(*) FROM " . idf_escape($row["Name"]));
 			$return[$row["Name"]] = $row;
 		}
 		foreach (get_rows("SELECT * FROM sqlite_sequence", null, "") as $row) {
@@ -259,47 +284,91 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 	function is_view($table_status) {
 		return $table_status["Engine"] == "view";
 	}
-	
+
 	function fk_support($table_status) {
 		global $connection;
-		return $_GET["create"] == "" && !$connection->result("SELECT sqlite_compileoption_used('OMIT_FOREIGN_KEY')");
+		return !$connection->result("SELECT sqlite_compileoption_used('OMIT_FOREIGN_KEY')");
 	}
 
 	function fields($table) {
+		global $connection;
 		$return = array();
+		$primary = "";
 		foreach (get_rows("PRAGMA table_info(" . table($table) . ")") as $row) {
+			$name = $row["name"];
 			$type = strtolower($row["type"]);
 			$default = $row["dflt_value"];
-			$return[$row["name"]] = array(
-				"field" => $row["name"],
-				"type" => (eregi("int", $type) ? "integer" : (eregi("char|clob|text", $type) ? "text" : (eregi("blob", $type) ? "blob" : (eregi("real|floa|doub", $type) ? "real" : "numeric")))),
+			$return[$name] = array(
+				"field" => $name,
+				"type" => (preg_match('~int~i', $type) ? "integer" : (preg_match('~char|clob|text~i', $type) ? "text" : (preg_match('~blob~i', $type) ? "blob" : (preg_match('~real|floa|doub~i', $type) ? "real" : "numeric")))),
 				"full_type" => $type,
-				"default" => (ereg("'(.*)'", $default, $match) ? str_replace("''", "'", $match[1]) : ($default == "NULL" ? null : $default)),
+				"default" => (preg_match("~'(.*)'~", $default, $match) ? str_replace("''", "'", $match[1]) : ($default == "NULL" ? null : $default)),
 				"null" => !$row["notnull"],
-				"auto_increment" => eregi('^integer$', $type) && $row["pk"], //! possible false positive
 				"privileges" => array("select" => 1, "insert" => 1, "update" => 1),
 				"primary" => $row["pk"],
 			);
+			if ($row["pk"]) {
+				if ($primary != "") {
+					$return[$primary]["auto_increment"] = false;
+				} elseif (preg_match('~^integer$~i', $type)) {
+					$return[$name]["auto_increment"] = true;
+				}
+				$primary = $name;
+			}
+		}
+		$sql = $connection->result("SELECT sql FROM sqlite_master WHERE type = 'table' AND name = " . q($table));
+		preg_match_all('~(("[^"]*+")+|[a-z0-9_]+)\s+text\s+COLLATE\s+(\'[^\']+\'|\S+)~i', $sql, $matches, PREG_SET_ORDER);
+		foreach ($matches as $match) {
+			$name = str_replace('""', '"', preg_replace('~^"|"$~', '', $match[1]));
+			if ($return[$name]) {
+				$return[$name]["collation"] = trim($match[3], "'");
+			}
 		}
 		return $return;
 	}
 
 	function indexes($table, $connection2 = null) {
+		global $connection;
+		if (!is_object($connection2)) {
+			$connection2 = $connection;
+		}
 		$return = array();
-		$primary = array();
-		foreach (fields($table) as $field) {
-			if ($field["primary"]) {
-				$primary[] = $field["field"];
+		$sql = $connection2->result("SELECT sql FROM sqlite_master WHERE type = 'table' AND name = " . q($table));
+		if (preg_match('~\bPRIMARY\s+KEY\s*\((([^)"]+|"[^"]*")++)~i', $sql, $match)) {
+			$return[""] = array("type" => "PRIMARY", "columns" => array(), "lengths" => array(), "descs" => array());
+			preg_match_all('~((("[^"]*+")+)|(\S+))(\s+(ASC|DESC))?(,\s*|$)~i', $match[1], $matches, PREG_SET_ORDER);
+			foreach ($matches as $match) {
+				$return[""]["columns"][] = idf_unescape($match[2]) . $match[4];
+				$return[""]["descs"][] = (preg_match('~DESC~i', $match[5]) ? '1' : null);
 			}
 		}
-		if ($primary) {
-			$return[""] = array("type" => "PRIMARY", "columns" => $primary, "lengths" => array());
+		if (!$return) {
+			foreach (fields($table) as $name => $field) {
+				if ($field["primary"]) {
+					$return[""] = array("type" => "PRIMARY", "columns" => array($name), "lengths" => array(), "descs" => array(null));
+				}
+			}
 		}
-		foreach (get_rows("PRAGMA index_list(" . table($table) . ")") as $row) {
-			$return[$row["name"]]["type"] = ($row["unique"] ? "UNIQUE" : "INDEX");
-			$return[$row["name"]]["lengths"] = array();
-			foreach (get_rows("PRAGMA index_info(" . idf_escape($row["name"]) . ")") as $row1) {
-				$return[$row["name"]]["columns"][] = $row1["name"];
+		$sqls = get_key_vals("SELECT name, sql FROM sqlite_master WHERE type = 'index' AND tbl_name = " . q($table), $connection2);
+		foreach (get_rows("PRAGMA index_list(" . table($table) . ")", $connection2) as $row) {
+			$name = $row["name"];
+			$index = array("type" => ($row["unique"] ? "UNIQUE" : "INDEX"));
+			$index["lengths"] = array();
+			$index["descs"] = array();
+			foreach (get_rows("PRAGMA index_info(" . idf_escape($name) . ")", $connection2) as $row1) {
+				$index["columns"][] = $row1["name"];
+				$index["descs"][] = null;
+			}
+			if (preg_match('~^CREATE( UNIQUE)? INDEX ' . preg_quote(idf_escape($name) . ' ON ' . idf_escape($table), '~') . ' \((.*)\)$~i', $sqls[$name], $regs)) {
+				preg_match_all('/("[^"]*+")+( DESC)?/', $regs[2], $matches);
+				foreach ($matches[2] as $key => $val) {
+					if ($val) {
+						$index["descs"][$key] = '1';
+					}
+				}
+			}
+			if (!$return[""] || $index["type"] != "UNIQUE" || $index["columns"] != $return[""]["columns"] || $index["descs"] != $return[""]["descs"] || !preg_match("~^sqlite_~", $name)) {
+				$return[$name] = $index;
 			}
 		}
 		return $return;
@@ -336,11 +405,7 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 		global $connection;
 		return h($connection->error);
 	}
-	
-	function exact_value($val) {
-		return q($val);
-	}
-	
+
 	function check_sqlite_name($name) {
 		// avoid creating PHP files on unsecured servers
 		global $connection;
@@ -351,7 +416,7 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 		}
 		return true;
 	}
-	
+
 	function create_database($db, $collation) {
 		global $connection;
 		if (file_exists($db)) {
@@ -361,13 +426,18 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 		if (!check_sqlite_name($db)) {
 			return false;
 		}
-		$link = new Min_SQLite($db); //! exception handler
+		try {
+			$link = new Min_SQLite($db);
+		} catch (Exception $ex) {
+			$connection->error = $ex->getMessage();
+			return false;
+		}
 		$link->query('PRAGMA encoding = "UTF-8"');
 		$link->query('CREATE TABLE adminer (i)'); // otherwise creates empty file
 		$link->query('DROP TABLE adminer');
 		return true;
 	}
-	
+
 	function drop_databases($databases) {
 		global $connection;
 		$connection->Min_SQLite(":memory:"); // to unlock file, doesn't work in PDO on Windows
@@ -379,7 +449,7 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 		}
 		return true;
 	}
-	
+
 	function rename_database($name, $collation) {
 		global $connection;
 		if (!check_sqlite_name($name)) {
@@ -389,20 +459,30 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 		$connection->error = lang('File exists.');
 		return @rename(DB, $name);
 	}
-	
+
 	function auto_increment() {
 		return " PRIMARY KEY" . (DRIVER == "sqlite" ? " AUTOINCREMENT" : "");
 	}
-	
+
 	function alter_table($table, $name, $fields, $foreign, $comment, $engine, $collation, $auto_increment, $partitioning) {
-		$alter = array();
+		$use_all_fields = ($table == "" || $foreign);
 		foreach ($fields as $field) {
-			if ($field[1]) {
-				$alter[] = ($table != "" && $field[0] == "" ? "ADD " : "  ") . implode($field[1]);
+			if ($field[0] != "" || !$field[1] || $field[2]) {
+				$use_all_fields = true;
+				break;
 			}
 		}
-		$alter = array_merge($alter, $foreign);
-		if ($table != "") {
+		$alter = array();
+		$originals = array();
+		foreach ($fields as $field) {
+			if ($field[1]) {
+				$alter[] = ($use_all_fields ? $field[1] : "ADD " . implode($field[1]));
+				if ($field[0] != "") {
+					$originals[$field[0]] = $field[1][0];
+				}
+			}
+		}
+		if (!$use_all_fields) {
 			foreach ($alter as $val) {
 				if (!queries("ALTER TABLE " . table($table) . " $val")) {
 					return false;
@@ -411,7 +491,7 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 			if ($table != $name && !queries("ALTER TABLE " . table($table) . " RENAME TO " . table($name))) {
 				return false;
 			}
-		} elseif (!queries("CREATE TABLE " . table($name) . " (\n" . implode(",\n", $alter) . "\n)")) {
+		} elseif (!recreate_table($table, $name, $alter, $originals, $foreign)) {
 			return false;
 		}
 		if ($auto_increment) {
@@ -419,128 +499,243 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 		}
 		return true;
 	}
-	
+
+	function recreate_table($table, $name, $fields, $originals, $foreign, $indexes = array()) {
+		if ($table != "") {
+			if (!$fields) {
+				foreach (fields($table) as $key => $field) {
+					$fields[] = process_field($field, $field);
+					$originals[$key] = idf_escape($key);
+				}
+			}
+			$primary_key = false;
+			foreach ($fields as $field) {
+				if ($field[6]) {
+					$primary_key = true;
+				}
+			}
+			$drop_indexes = array();
+			foreach ($indexes as $key => $val) {
+				if ($val[2] == "DROP") {
+					$drop_indexes[$val[1]] = true;
+					unset($indexes[$key]);
+				}
+			}
+			foreach (indexes($table) as $key_name => $index) {
+				$columns = array();
+				foreach ($index["columns"] as $key => $column) {
+					if (!$originals[$column]) {
+						continue 2;
+					}
+					$columns[] = $originals[$column] . ($index["descs"][$key] ? " DESC" : "");
+				}
+				if (!$drop_indexes[$key_name]) {
+					if ($index["type"] != "PRIMARY" || !$primary_key) {
+						$indexes[] = array($index["type"], $key_name, $columns);
+					}
+				}
+			}
+			foreach ($indexes as $key => $val) {
+				if ($val[0] == "PRIMARY") {
+					unset($indexes[$key]);
+					$foreign[] = "  PRIMARY KEY (" . implode(", ", $val[2]) . ")";
+				}
+			}
+			foreach (foreign_keys($table) as $key_name => $foreign_key) {
+				foreach ($foreign_key["source"] as $key => $column) {
+					if (!$originals[$column]) {
+						continue 2;
+					}
+					$foreign_key["source"][$key] = idf_unescape($originals[$column]);
+				}
+				if (!isset($foreign[" $key_name"])) {
+					$foreign[] = " " . format_foreign_key($foreign_key);
+				}
+			}
+			queries("BEGIN");
+		}
+		foreach ($fields as $key => $field) {
+			$fields[$key] = "  " . implode($field);
+		}
+		$fields = array_merge($fields, array_filter($foreign));
+		if (!queries("CREATE TABLE " . table($table != "" ? "adminer_$name" : $name) . " (\n" . implode(",\n", $fields) . "\n)")) {
+			// implicit ROLLBACK to not overwrite $connection->error
+			return false;
+		}
+		if ($table != "") {
+			if ($originals && !queries("INSERT INTO " . table("adminer_$name") . " (" . implode(", ", $originals) . ") SELECT " . implode(", ", array_map('idf_escape', array_keys($originals))) . " FROM " . table($table))) {
+				return false;
+			}
+			$triggers = array();
+			foreach (triggers($table) as $trigger_name => $timing_event) {
+				$trigger = trigger($trigger_name);
+				$triggers[] = "CREATE TRIGGER " . idf_escape($trigger_name) . " " . implode(" ", $timing_event) . " ON " . table($name) . "\n$trigger[Statement]";
+			}
+			if (!queries("DROP TABLE " . table($table))) { // drop before creating indexes and triggers to allow using old names
+				return false;
+			}
+			queries("ALTER TABLE " . table("adminer_$name") . " RENAME TO " . table($name));
+			if (!alter_indexes($name, $indexes)) {
+				return false;
+			}
+			foreach ($triggers as $trigger) {
+				if (!queries($trigger)) {
+					return false;
+				}
+			}
+			queries("COMMIT");
+		}
+		return true;
+	}
+
+	function index_sql($table, $type, $name, $columns) {
+		return "CREATE $type " . ($type != "INDEX" ? "INDEX " : "")
+			. idf_escape($name != "" ? $name : uniqid($table . "_"))
+			. " ON " . table($table)
+			. " $columns"
+		;
+	}
+
 	function alter_indexes($table, $alter) {
-		foreach ($alter as $val) {
+		foreach ($alter as $primary) {
+			if ($primary[0] == "PRIMARY") {
+				return recreate_table($table, $table, array(), array(), array(), $alter);
+			}
+		}
+		foreach (array_reverse($alter) as $val) {
 			if (!queries($val[2] == "DROP"
 				? "DROP INDEX " . idf_escape($val[1])
-				: "CREATE $val[0] " . ($val[0] != "INDEX" ? "INDEX " : "") . idf_escape($val[1] != "" ? $val[1] : uniqid($table . "_")) . " ON " . table($table) . " $val[2]"
+				: index_sql($table, $val[0], $val[1], "(" . implode(", ", $val[2]) . ")")
 			)) {
 				return false;
 			}
 		}
 		return true;
 	}
-	
+
 	function truncate_tables($tables) {
 		return apply_queries("DELETE FROM", $tables);
 	}
-	
+
 	function drop_views($views) {
 		return apply_queries("DROP VIEW", $views);
 	}
-	
+
 	function drop_tables($tables) {
 		return apply_queries("DROP TABLE", $tables);
 	}
-	
+
 	function move_tables($tables, $views, $target) {
 		return false;
 	}
-	
+
 	function trigger($name) {
 		global $connection;
 		if ($name == "") {
 			return array("Statement" => "BEGIN\n\t;\nEND");
 		}
-		preg_match('~^CREATE\\s+TRIGGER\\s*(?:[^`"\\s]+|`[^`]*`|"[^"]*")+\\s*([a-z]+)\\s+([a-z]+)\\s+ON\\s*(?:[^`"\\s]+|`[^`]*`|"[^"]*")+\\s*(?:FOR\\s*EACH\\s*ROW\\s)?(.*)~is', $connection->result("SELECT sql FROM sqlite_master WHERE name = " . q($name)), $match);
-		return array("Timing" => strtoupper($match[1]), "Event" => strtoupper($match[2]), "Trigger" => $name, "Statement" => $match[3]);
+		$idf = '(?:[^`"\\s]+|`[^`]*`|"[^"]*")+';
+		$trigger_options = trigger_options();
+		preg_match(
+			"~^CREATE\\s+TRIGGER\\s*$idf\\s*(" . implode("|", $trigger_options["Timing"]) . ")\\s+([a-z]+)(?:\\s+OF\\s+($idf))?\\s+ON\\s*$idf\\s*(?:FOR\\s+EACH\\s+ROW\\s)?(.*)~is",
+			$connection->result("SELECT sql FROM sqlite_master WHERE type = 'trigger' AND name = " . q($name)),
+			$match
+		);
+		$of = $match[3];
+		return array(
+			"Timing" => strtoupper($match[1]),
+			"Event" => strtoupper($match[2]) . ($of ? " OF" : ""),
+			"Of" => ($of[0] == '`' || $of[0] == '"' ? idf_unescape($of) : $of),
+			"Trigger" => $name,
+			"Statement" => $match[4],
+		);
 	}
-	
+
 	function triggers($table) {
 		$return = array();
+		$trigger_options = trigger_options();
 		foreach (get_rows("SELECT * FROM sqlite_master WHERE type = 'trigger' AND tbl_name = " . q($table)) as $row) {
-			preg_match('~^CREATE\\s+TRIGGER\\s*(?:[^`"\\s]+|`[^`]*`|"[^"]*")+\\s*([a-z]+)\\s*([a-z]+)~i', $row["sql"], $match);
+			preg_match('~^CREATE\\s+TRIGGER\\s*(?:[^`"\\s]+|`[^`]*`|"[^"]*")+\\s*(' . implode("|", $trigger_options["Timing"]) . ')\\s*(.*)\\s+ON\\b~iU', $row["sql"], $match);
 			$return[$row["name"]] = array($match[1], $match[2]);
 		}
 		return $return;
 	}
-	
+
 	function trigger_options() {
 		return array(
 			"Timing" => array("BEFORE", "AFTER", "INSTEAD OF"),
+			"Event" => array("INSERT", "UPDATE", "UPDATE OF", "DELETE"),
 			"Type" => array("FOR EACH ROW"),
 		);
 	}
-	
+
 	function routine($name, $type) {
 		// not supported by SQLite
 	}
-	
+
 	function routines() {
 		// not supported by SQLite
 	}
-	
+
 	function routine_languages() {
 		// not supported by SQLite
 	}
-	
+
 	function begin() {
 		return queries("BEGIN");
 	}
-	
-	function insert_into($table, $set) {
-		return queries("INSERT INTO " . table($table) . ($set ? " (" . implode(", ", array_keys($set)) . ")\nVALUES (" . implode(", ", $set) . ")" : "DEFAULT VALUES"));
-	}
-	
-	function insert_update($table, $set, $primary) {
-		return queries("REPLACE INTO " . table($table) . " (" . implode(", ", array_keys($set)) . ") VALUES (" . implode(", ", $set) . ")");
-	}
-	
+
 	function last_id() {
 		global $connection;
 		return $connection->result("SELECT LAST_INSERT_ROWID()");
 	}
-	
+
 	function explain($connection, $query) {
-		return $connection->query("EXPLAIN $query");
+		return $connection->query("EXPLAIN QUERY PLAN $query");
 	}
-	
+
 	function found_rows($table_status, $where) {
 	}
-	
+
 	function types() {
 		return array();
 	}
-	
+
 	function schemas() {
 		return array();
 	}
-	
+
 	function get_schema() {
 		return "";
 	}
-	
+
 	function set_schema($scheme) {
 		return true;
 	}
-	
+
 	function create_sql($table, $auto_increment) {
 		global $connection;
-		return $connection->result("SELECT sql FROM sqlite_master WHERE type = 'table' AND name = " . q($table));
+		$return = $connection->result("SELECT sql FROM sqlite_master WHERE type IN ('table', 'view') AND name = " . q($table));
+		foreach (indexes($table) as $name => $index) {
+			if ($name == '') {
+				continue;
+			}
+			$return .= ";\n\n" . index_sql($table, $index['type'], $name, "(" . implode(", ", array_map('idf_escape', $index['columns'])) . ")");
+		}
+		return $return;
 	}
-	
+
 	function truncate_sql($table) {
 		return "DELETE FROM " . table($table);
 	}
-	
+
 	function use_sql($database) {
 	}
-	
+
 	function trigger_sql($table, $style) {
 		return implode(get_vals("SELECT sql || ';;\n' FROM sqlite_master WHERE type = 'trigger' AND tbl_name = " . q($table)));
 	}
-	
+
 	function show_variables() {
 		global $connection;
 		$return = array();
@@ -549,7 +744,7 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 		}
 		return $return;
 	}
-	
+
 	function show_status() {
 		$return = array();
 		foreach (get_vals("PRAGMA compile_options") as $option) {
@@ -558,16 +753,23 @@ if (isset($_GET["sqlite"]) || isset($_GET["sqlite2"])) {
 		}
 		return $return;
 	}
-	
-	function support($feature) {
-		return ereg('^(view|trigger|variables|status|dump)$', $feature);
+
+	function convert_field($field) {
 	}
-	
+
+	function unconvert_field($field, $return) {
+		return $return;
+	}
+
+	function support($feature) {
+		return preg_match('~^(columns|database|drop_col|dump|indexes|move_col|sql|status|table|trigger|variables|view|view_trigger)$~', $feature);
+	}
+
 	$jush = "sqlite";
 	$types = array("integer" => 0, "real" => 0, "numeric" => 0, "text" => 0, "blob" => 0);
 	$structured_types = array_keys($types);
 	$unsigned = array();
-	$operators = array("=", "<", ">", "<=", ">=", "!=", "LIKE", "LIKE %%", "IN", "IS NULL", "NOT LIKE", "NOT IN", "IS NOT NULL", ""); // REGEXP can be user defined function
+	$operators = array("=", "<", ">", "<=", ">=", "!=", "LIKE", "LIKE %%", "IN", "IS NULL", "NOT LIKE", "NOT IN", "IS NOT NULL", "SQL"); // REGEXP can be user defined function
 	$functions = array("hex", "length", "lower", "round", "unixepoch", "upper");
 	$grouping = array("avg", "count", "count distinct", "group_concat", "max", "min", "sum");
 	$edit_functions = array(
