@@ -8,14 +8,14 @@ $translations = array(
 	'Password' => 'Passwort',
 	'Select database' => 'Datenbank auswählen',
 	'Invalid database.' => 'Datenbank ungültig.',
-	'Create new database' => 'Neue Datenbank',
-	'Table has been dropped.' => 'Tabelle entfernt.',
-	'Table has been altered.' => 'Tabelle geändert.',
-	'Table has been created.' => 'Tabelle erstellt.',
+	'Create new database' => 'Datenbank erstellen',
+	'Table has been dropped.' => 'Tabelle wurde entfernt.',
+	'Table has been altered.' => 'Tabelle wurde geändert.',
+	'Table has been created.' => 'Tabelle wurde erstellt.',
 	'Alter table' => 'Tabelle ändern',
-	'Create table' => 'Neue Tabelle erstellen',
+	'Create table' => 'Tabelle erstellen',
 	'Table name' => 'Name der Tabelle',
-	'engine' => 'Motor',
+	'engine' => 'Speicher-Engine',
 	'collation' => 'Kollation',
 	'Column name' => 'Spaltenname',
 	'Type' => 'Typ',
@@ -24,29 +24,27 @@ $translations = array(
 	'Options' => 'Optionen',
 	'Save' => 'Speichern',
 	'Drop' => 'Entfernen',
-	'Database has been dropped.' => 'Datenbank entfernt.',
-	'Database has been created.' => 'Datenbank erstellt.',
-	'Database has been renamed.' => 'Datenbank umbenannt.',
-	'Database has been altered.' => 'Datenbank geändert.',
+	'Database has been dropped.' => 'Datenbank wurde entfernt.',
+	'Database has been created.' => 'Datenbank wurde erstellt.',
+	'Database has been renamed.' => 'Datenbank wurde umbenannt.',
+	'Database has been altered.' => 'Datenbank wurde geändert.',
 	'Alter database' => 'Datenbank ändern',
-	'Create database' => 'Neue Datenbank',
-	'SQL command' => 'SQL-Query',
-	'Dump' => 'Export',
+	'Create database' => 'Datenbank erstellen',
+	'SQL command' => 'SQL-Kommando',
 	'Logout' => 'Abmelden',
 	'database' => 'Datenbank',
 	'Use' => 'Benutzung',
 	'No tables.' => 'Keine Tabellen.',
 	'select' => 'zeigen',
-	'Create new table' => 'Neue Tabelle',
-	'Item has been deleted.' => 'Datensatz gelöscht.',
-	'Item has been updated.' => 'Datensatz geändert.',
-	'Item%s has been inserted.' => 'Datensatz%s hinzugefügt.',
-	'Edit' => 'Ändern',
-	'Insert' => 'Hinzufügen',
-	'Save and insert next' => 'Speichern und nächsten hinzufügen',
+	'Item has been deleted.' => 'Datensatz wurde gelöscht.',
+	'Item has been updated.' => 'Datensatz wurde geändert.',
+	'Item%s has been inserted.' => 'Datensatz%s wurde eingefügt.',
+	'Edit' => 'Bearbeiten',
+	'Insert' => 'Einfügen',
+	'Save and insert next' => 'Speichern und nächsten einfügen',
 	'Delete' => 'Entfernen',
 	'Database' => 'Datenbank',
-	'Routines' => 'Prozeduren',
+	'Routines' => 'Routinen',
 	'Indexes have been altered.' => 'Indizes geändert.',
 	'Indexes' => 'Indizes',
 	'Alter indexes' => 'Indizes ändern',
@@ -58,9 +56,9 @@ $translations = array(
 	'Sort' => 'Ordnen',
 	'descending' => 'absteigend',
 	'Limit' => 'Begrenzung',
-	'No rows.' => 'Keine Daten.',
+	'No rows.' => 'Keine Datensätze.',
 	'Action' => 'Aktion',
-	'edit' => 'ändern',
+	'edit' => 'bearbeiten',
 	'Page' => 'Seite',
 	'Query executed OK, %d row(s) affected.' => array('Abfrage ausgeführt, %d Datensatz betroffen.', 'Abfrage ausgeführt, %d Datensätze betroffen.'),
 	'Error in query' => 'Fehler in der SQL-Abfrage',
@@ -78,16 +76,16 @@ $translations = array(
 	'Unable to upload a file.' => 'Hochladen von Datei fehlgeschlagen.',
 	'File upload' => 'Datei importieren',
 	'File uploads are disabled.' => 'Importieren von Dateien abgeschaltet.',
-	'Routine has been called, %d row(s) affected.' => array('Kommando SQL ausgeführt, %d Datensatz betroffen.', 'Kommando SQL ausgeführt, %d Datensätze betroffen.'),
+	'Routine has been called, %d row(s) affected.' => array('Routine wurde ausgeführt, %d Datensatz betroffen.', 'Routine wurde ausgeführt, %d Datensätze betroffen.'),
 	'Call' => 'Aufrufen',
 	'No extension' => 'Keine Erweiterungen installiert',
 	'None of the supported PHP extensions (%s) are available.' => 'Keine der unterstützten PHP-Erweiterungen (%s) ist vorhanden.',
-	'Session support must be enabled.' => 'Sitzungen müssen aktiviert sein.',
+	'Session support must be enabled.' => 'Unterstüzung für PHP-Sessions muss aktiviert sein.',
 	'Session expired, please login again.' => 'Sitzungsdauer abgelaufen, bitte erneut anmelden.',
 	'Text length' => 'Textlänge',
-	'Foreign key has been dropped.' => 'Fremdschlüssel entfernt.',
-	'Foreign key has been altered.' => 'Fremdschlüssel geändert.',
-	'Foreign key has been created.' => 'Fremdschlüssel erstellt.',
+	'Foreign key has been dropped.' => 'Fremdschlüssel wurde entfernt.',
+	'Foreign key has been altered.' => 'Fremdschlüssel wurde geändert.',
+	'Foreign key has been created.' => 'Fremdschlüssel wurde erstellt.',
 	'Foreign key' => 'Fremdschlüssel',
 	'Target table' => 'Zieltabelle',
 	'Change' => 'Ändern',
@@ -100,53 +98,53 @@ $translations = array(
 	'ON UPDATE' => 'ON UPDATE',
 	'Index Type' => 'Index-Typ',
 	'Column (length)' => 'Spalte (Länge)',
-	'View has been dropped.' => 'View entfernt.',
-	'View has been altered.' => 'View geändert.',
-	'View has been created.' => 'View erstellt.',
+	'View has been dropped.' => 'View wurde entfernt.',
+	'View has been altered.' => 'View wurde geändert.',
+	'View has been created.' => 'View wurde erstellt.',
 	'Alter view' => 'View ändern',
-	'Create view' => 'Neue View erstellen',
+	'Create view' => 'View erstellen',
 	'Name' => 'Name',
 	'Process list' => 'Prozessliste',
 	'%d process(es) have been killed.' => array('%d Prozess gestoppt.', '%d Prozesse gestoppt.'),
 	'Kill' => 'Anhalten',
 	'Parameter name' => 'Name des Parameters',
 	'Database schema' => 'Datenbankschema',
-	'Create procedure' => 'Neue Prozedur',
-	'Create function' => 'Neue Funktion',
-	'Routine has been dropped.' => 'Prozedur entfernt.',
-	'Routine has been altered.' => 'Prozedur geändert.',
-	'Routine has been created.' => 'Prozedur erstellt.',
+	'Create procedure' => 'Prozedur erstellen',
+	'Create function' => 'Funktion erstellen',
+	'Routine has been dropped.' => 'Routine wurde entfernt.',
+	'Routine has been altered.' => 'Routine wurde geändert.',
+	'Routine has been created.' => 'Routine wurde erstellt.',
 	'Alter function' => 'Funktion ändern',
 	'Alter procedure' => 'Prozedur ändern',
 	'Return type' => 'Typ des Rückgabewertes',
 	'Add trigger' => 'Trigger hinzufügen',
-	'Trigger has been dropped.' => 'Trigger entfernt.',
-	'Trigger has been altered.' => 'Trigger geändert.',
-	'Trigger has been created.' => 'Trigger erstellt.',
+	'Trigger has been dropped.' => 'Trigger wurde entfernt.',
+	'Trigger has been altered.' => 'Trigger wurde geändert.',
+	'Trigger has been created.' => 'Trigger wurde erstellt.',
 	'Alter trigger' => 'Trigger ändern',
-	'Create trigger' => 'Trigger hinzufügen',
+	'Create trigger' => 'Trigger erstellen',
 	'Time' => 'Zeitpunkt',
 	'Event' => 'Ereignis',
-	'%s version: %s through PHP extension %s' => 'Version %s: %s, mit PHP-Erweiterung %s',
+	'%s version: %s through PHP extension %s' => 'Version %s: %s mit PHP-Erweiterung %s',
 	'%d row(s)' => array('%d Datensatz', '%d Datensätze'),
 	'Remove' => 'Entfernen',
-	'Are you sure?' => 'Sind Sie sicher ?',
+	'Are you sure?' => 'Sind Sie sicher?',
 	'Privileges' => 'Rechte',
-	'Create user' => 'Neuer Benutzer',
-	'User has been dropped.' => 'Benutzer entfernt.',
-	'User has been altered.' => 'Benutzer geändert.',
-	'User has been created.' => 'Benutzer erstellt.',
+	'Create user' => 'Benutzer erstellen',
+	'User has been dropped.' => 'Benutzer wurde entfernt.',
+	'User has been altered.' => 'Benutzer wurde geändert.',
+	'User has been created.' => 'Benutzer wurde erstellt.',
 	'Hashed' => 'Hashed',
 	'Column' => 'Spalte',
-	'Routine' => 'Rutine',
+	'Routine' => 'Routine',
 	'Grant' => 'Erlauben',
-	'Revoke' => 'Verbieten',
-	'Too big POST data. Reduce the data or increase the %s configuration directive.' => 'POST data zu gross. Reduzieren Sie die Grösse oder vergrössern Sie den Wert %s in der Konfiguration.',
+	'Revoke' => 'Widerrufen',
+	'Too big POST data. Reduce the data or increase the %s configuration directive.' => 'POST-Daten sind zu groß. Reduzieren Sie die Größe oder vergrößern Sie den Wert %s in der Konfiguration.',
 	'Logged as: %s' => 'Angemeldet als: %s',
 	'Move up' => 'Nach oben',
 	'Move down' => 'Nach unten',
 	'Functions' => 'Funktionen',
-	'Aggregation' => 'Agregationen',
+	'Aggregation' => 'Aggregationen',
 	'Export' => 'Exportieren',
 	'Output' => 'Ergebnis',
 	'open' => 'anzeigen',
@@ -154,9 +152,9 @@ $translations = array(
 	'Format' => 'Format',
 	'Tables' => 'Tabellen',
 	'Data' => 'Daten',
-	'Event has been dropped.' => 'Ereignis entfernt.',
-	'Event has been altered.' => 'Ereignis geändert.',
-	'Event has been created.' => 'Ereignis erstellt.',
+	'Event has been dropped.' => 'Ereignis wurde entfernt.',
+	'Event has been altered.' => 'Ereignis wurde geändert.',
+	'Event has been created.' => 'Ereignis wurde erstellt.',
 	'Alter event' => 'Ereignis ändern',
 	'Create event' => 'Ereignis erstellen',
 	'At given time' => 'Zur angegebenen Zeit',
@@ -168,34 +166,35 @@ $translations = array(
 	'Status' => 'Status',
 	'On completion preserve' => 'Nach der Ausführung erhalten',
 	'Tables and views' => 'Tabellen und Views',
-	'Data Length' => 'Datengrösse',
-	'Index Length' => 'Indexgrösse',
+	'Data Length' => 'Datengröße',
+	'Index Length' => 'Indexgröße',
 	'Data Free' => 'Freier Bereich',
-	'Collation' => 'Collation',
+	'Collation' => 'Kollation',
 	'Analyze' => 'Analysieren',
 	'Optimize' => 'Optimieren',
 	'Check' => 'Prüfen',
 	'Repair' => 'Reparieren',
-	'Truncate' => 'Entleeren (truncate)',
-	'Tables have been truncated.' => 'Tabellen sind entleert worden (truncate).',
+	'Truncate' => 'Leeren (truncate)',
+	'Tables have been truncated.' => 'Tabellen wurden geleert (truncate).',
 	'Rows' => 'Datensätze',
 	',' => ' ',
+	'0123456789' => '0123456789',
 	'Tables have been moved.' => 'Tabellen verschoben.',
 	'Move to other database' => 'In andere Datenbank verschieben',
 	'Move' => 'Verschieben',
-	'Engine' => 'Motor',
+	'Engine' => 'Speicher-Engine',
 	'Save and continue edit' => 'Speichern und weiter bearbeiten',
 	'original' => 'Original',
 	'Tables have been dropped.' => 'Tabellen wurden entfernt (drop).',
 	'%d item(s) have been affected.' => '%d Artikel betroffen.',
 	'whole result' => 'Gesamtergebnis',
 	'Clone' => 'Klonen',
-	'Maximum number of allowed fields exceeded. Please increase %s and %s.' => 'Die maximal erlaubte Anzahl der Felder ist überschritten. Bitte %s und %s erhöhen.',
+	'Maximum number of allowed fields exceeded. Please increase %s.' => 'Die maximal erlaubte Anzahl der Felder ist überschritten. Bitte %s erhöhen.',
 	'Partition by' => 'Partitionieren um',
 	'Partitions' => 'Partitionen',
 	'Partition name' => 'Name der Partition',
 	'Values' => 'Werte',
-	'%d row(s) have been imported.' => array('%d Datensatz importiert.', '%d Datensätze wurden importiert.'),
+	'%d row(s) have been imported.' => array('%d Datensatz wurde importiert.', '%d Datensätze wurden importiert.'),
 	'anywhere' => 'beliebig',
 	'Import' => 'Importieren',
 	'Stop on error' => 'Bei Fehler anhalten',
@@ -204,50 +203,49 @@ $translations = array(
 	'[yyyy]-mm-dd' => 't.m.[jjjj]',
 	'History' => 'History',
 	'Variables' => 'Variablen',
-	'Source and target columns must have the same data type, there must be an index on the target columns and referenced data must exist.' => 'Spalten des Ursprungs und des Zieles müssen vom gleichen Datentyp sein, es muss unter den Zielspalten ein Index existieren und die referenzierten Daten müssen existieren.',
+	'Source and target columns must have the same data type, there must be an index on the target columns and referenced data must exist.' => 'Quell- und Zielspalten müssen vom gleichen Datentyp sein, es muss unter den Zielspalten ein Index existieren und die referenzierten Daten müssen existieren.',
 	'Relations' => 'Relationen',
 	'Run file' => 'Datei ausführen',
-	'Clear' => 'Entleeren',
-	'Maximum allowed file size is %sB.' => 'Maximal erlaubte Dateigrösse ist %sB.',
+	'Clear' => 'Löschen',
+	'Maximum allowed file size is %sB.' => 'Maximal erlaubte Dateigröße ist %sB.',
 	'Numbers' => 'Zahlen',
-	'Date and time' => 'Datum oder Zeit',
+	'Date and time' => 'Datum und Zeit',
 	'Strings' => 'Zeichenketten',
 	'Binary' => 'Binär',
 	'Lists' => 'Listen',
 	'Editor' => 'Editor',
-	'E-mail' => 'E-mail',
+	'E-mail' => 'E-Mail',
 	'From' => 'Von',
 	'Subject' => 'Betreff',
 	'Send' => 'Abschicken',
-	'%d e-mail(s) have been sent.' => array('%d e-mail abgeschickt.', '%d e-mails abgeschickt.'),
+	'%d e-mail(s) have been sent.' => array('%d E-Mail abgeschickt.', '%d E-Mails abgeschickt.'),
 	'Webserver file %s' => 'Webserver Datei %s',
 	'File does not exist.' => 'Datei existiert nicht.',
 	'%d in total' => '%d insgesamt',
 	'Permanent login' => 'Passwort speichern',
-	'Databases have been dropped.' => 'Datenbanken entfernt.',
+	'Databases have been dropped.' => 'Datenbanken wurden entfernt.',
 	'Search data in tables' => 'Suche in Tabellen',
-	'schema' => 'Schema',
 	'Schema' => 'Schema',
 	'Alter schema' => 'Schema ändern',
-	'Create schema' => 'Neues Schema',
+	'Create schema' => 'Schema erstellen',
 	'Schema has been dropped.' => 'Schema wurde gelöscht.',
-	'Schema has been created.' => 'Neues Schema erstellt.',
-	'Schema has been altered.' => 'Schema geändert.',
-	'Sequences' => 'Sequenz',
-	'Create sequence' => 'Neue Sequenz',
+	'Schema has been created.' => 'Schema wurde erstellt.',
+	'Schema has been altered.' => 'Schema wurde geändert.',
+	'Sequences' => 'Sequenzen',
+	'Create sequence' => 'Sequenz erstellen',
 	'Alter sequence' => 'Sequenz ändern',
-	'Sequence has been dropped.' => 'Sequenz gelöscht.',
-	'Sequence has been created.' => 'Neue Sequenz erstellt.',
-	'Sequence has been altered.' => 'Sequenz geändert.',
-	'User types' => 'Benutzer-definierte Typen',
+	'Sequence has been dropped.' => 'Sequenz wurde gelöscht.',
+	'Sequence has been created.' => 'Sequenz wurde erstellt.',
+	'Sequence has been altered.' => 'Sequenz wurde geändert.',
+	'User types' => 'Benutzerdefinierte Typen',
 	'Create type' => 'Typ erstellen',
 	'Alter type' => 'Typ ändern',
-	'Type has been dropped.' => 'Typ gelöscht.',
-	'Type has been created.' => 'Typ erstellt.',
-	'Double click on a value to modify it.' => 'Doppelklick zum Bearbeiten des Wertes.',
-	'Use edit link to modify this value.' => 'Benutzen Sie den Link zum editieren dieses Wertes.',
+	'Type has been dropped.' => 'Typ wurde gelöscht.',
+	'Type has been created.' => 'Typ wurde erstellt.',
+	'Ctrl+click on a value to modify it.' => 'Ctrl+Klick zum Bearbeiten des Wertes.',
+	'Use edit link to modify this value.' => 'Benutzen Sie den Link zum Bearbeiten dieses Wertes.',
 	'last' => 'letzte',
-	'From server' => 'Im Server',
+	'From server' => 'Vom Server',
 	'System' => 'Datenbank System',
 	'Select data' => 'Daten auswählen',
 	'Show structure' => 'Struktur anzeigen',
@@ -256,9 +254,8 @@ $translations = array(
 	'Geometry' => 'Geometrie',
 	'File exists.' => 'Datei existiert schon.',
 	'Attachments' => 'Anhänge',
-	'%d query(s) executed OK.' => array('SQL-Query erfolgreich ausgeführt.', '%d SQL-Queries erfolgreich ausgeführt.'),
+	'%d query(s) executed OK.' => array('SQL-Abfrage erfolgreich ausgeführt.', '%d SQL-Abfragen erfolgreich ausgeführt.'),
 	'Show only errors' => 'Nur Fehler anzeigen',
-	'Last page' => 'Letzte Seite',
 	'Refresh' => 'Aktualisieren',
 	'Invalid schema.' => 'Schema nicht gültig.',
 	'Please use one of the extensions %s.' => 'Bitte einen der Dateitypen %s benutzen.',
@@ -267,7 +264,26 @@ $translations = array(
 	'Tables have been copied.' => 'Tabellen wurden kopiert.',
 	'Copy' => 'Kopieren',
 	'Permanent link' => 'Dauerhafter Link',
-	'Edit all' => 'Alle ändern',
-	'Resend POST data?' => 'POST data noch einmal senden ?',
+	'Edit all' => 'Alle bearbeiten',
 	'HH:MM:SS' => 'HH:MM:SS',
+	'Compute' => 'kalkulieren',
+	'Size' => 'Größe',
+	'Modify' => 'Ändern',
+	'Selected' => 'Ausgewählte',
+	'Default value' => 'Vorgabewert festlegen',
+	'Limit rows' => 'Datensätze begrenzen',
+	'Tables have been optimized.' => 'Tabellen wurden optimiert.',
+	'File must be in UTF-8 encoding.' => 'Die Datei muss UTF-8 kodiert sein.',
+	'Loading' => 'Lade',
+	'Load more data' => 'Mehr Daten laden',
+	'Too many unsuccessful logins, try again in %d minute(s).' => array('Zu viele erfolglose Login-Versuche. Bitte probieren Sie es in %d Minute noch einmal.', 'Zu viele erfolglose Login-Versuche. Bitte probieren Sie es in %d Minuten noch einmal.'),
+	'If you did not send this request from Adminer then close this page.' => 'Wenn Sie diese Anfrage nicht von Adminer gesendet haben, schließen Sie diese Seite.',
+	'You can upload a big SQL file via FTP and import it from server.' => 'Sie können eine große SQL-Datei per FTP hochladen und dann vom Server importieren.',
+	'You are offline.' => 'Sie sind offline.',
+	'You have no privileges to update this table.' => 'Sie haben keine Rechte, um diese Tabelle zu aktualisieren.' ,
+	'Saving' => 'Speichere',
+	'yes' => 'ja',
+	'no' => 'nein',
+	'Master password expired. <a href="http://www.adminer.org/en/extension/" target="_blank">Implement</a> %s method to make it permanent.' => 'Das Master-Passwort ist abgelaufen.  <a href="http://www.adminer.org/de/extension/" target="_blank">Implementieren</a> Sie die %s Methode, um es permanent zu machen.',
+	'%d / ' => '%d / ',
 );
