@@ -1,14 +1,17 @@
 <?php
+$row = $_POST;
+
 if ($_POST && !$error) {
 	$link = preg_replace('~ns=[^&]*&~', '', ME) . "ns=";
 	if ($_POST["drop"]) {
 		query_redirect("DROP SCHEMA " . idf_escape($_GET["ns"]), $link, lang('Schema has been dropped.'));
 	} else {
-		$link .= urlencode($_POST["name"]);
+		$name = trim($row["name"]);
+		$link .= urlencode($name);
 		if ($_GET["ns"] == "") {
-			query_redirect("CREATE SCHEMA " . idf_escape($_POST["name"]), $link, lang('Schema has been created.'));
-		} elseif ($_GET["ns"] != $_POST["name"]) {
-			query_redirect("ALTER SCHEMA " . idf_escape($_GET["ns"]) . " RENAME TO " . idf_escape($_POST["name"]), $link, lang('Schema has been altered.')); //! sp_rename in MS SQL
+			query_redirect("CREATE SCHEMA " . idf_escape($name), $link, lang('Schema has been created.'));
+		} elseif ($_GET["ns"] != $name) {
+			query_redirect("ALTER SCHEMA " . idf_escape($_GET["ns"]) . " RENAME TO " . idf_escape($name), $link, lang('Schema has been altered.')); //! sp_rename in MS SQL
 		} else {
 			redirect($link);
 		}
@@ -17,14 +20,14 @@ if ($_POST && !$error) {
 
 page_header($_GET["ns"] != "" ? lang('Alter schema') : lang('Create schema'), $error);
 
-$row = $_POST;
 if (!$row) {
-	$row = array("name" => $_GET["ns"]);
+	$row["name"] = $_GET["ns"];
 }
 ?>
 
 <form action="" method="post">
-<p><input name="name" value="<?php echo h($row["name"]); ?>">
+<p><input name="name" id="name" value="<?php echo h($row["name"]); ?>" autocapitalize="off">
+<script type='text/javascript'>focus(document.getElementById('name'));</script>
 <input type="submit" value="<?php echo lang('Save'); ?>">
 <?php
 if ($_GET["ns"] != "") {
