@@ -1,14 +1,16 @@
 <?php
 $SEQUENCE = $_GET["sequence"];
+$row = $_POST;
 
 if ($_POST && !$error) {
 	$link = substr(ME, 0, -1);
+	$name = trim($row["name"]);
 	if ($_POST["drop"]) {
 		query_redirect("DROP SEQUENCE " . idf_escape($SEQUENCE), $link, lang('Sequence has been dropped.'));
 	} elseif ($SEQUENCE == "") {
-		query_redirect("CREATE SEQUENCE " . idf_escape($_POST["name"]), $link, lang('Sequence has been created.'));
-	} elseif ($SEQUENCE != $_POST["name"]) {
-		query_redirect("ALTER SEQUENCE " . idf_escape($SEQUENCE) . " RENAME TO " . idf_escape($_POST["name"]), $link, lang('Sequence has been altered.'));
+		query_redirect("CREATE SEQUENCE " . idf_escape($name), $link, lang('Sequence has been created.'));
+	} elseif ($SEQUENCE != $name) {
+		query_redirect("ALTER SEQUENCE " . idf_escape($SEQUENCE) . " RENAME TO " . idf_escape($name), $link, lang('Sequence has been altered.'));
 	} else {
 		redirect($link);
 	}
@@ -16,14 +18,13 @@ if ($_POST && !$error) {
 
 page_header($SEQUENCE != "" ? lang('Alter sequence') . ": " . h($SEQUENCE) : lang('Create sequence'), $error);
 
-$row = $_POST;
 if (!$row) {
-	$row = array("name" => $SEQUENCE);
+	$row["name"] = $SEQUENCE;
 }
 ?>
 
 <form action="" method="post">
-<p><input name="name" value="<?php echo h($row["name"]); ?>">
+<p><input name="name" value="<?php echo h($row["name"]); ?>" autocapitalize="off">
 <input type="submit" value="<?php echo lang('Save'); ?>">
 <?php
 if ($SEQUENCE != "") {
