@@ -1,37 +1,37 @@
 // Adminer specific functions
 
-var jushRoot = '../externals/jush/'; // global variable to allow simple customization
-
 /** Load syntax highlighting
 * @param string first three characters of database system version
 */
 function bodyLoad(version) {
-	if (history.state !== undefined) { // copied from editor/static/editing.js
-		onpopstate(history);
-	}
-	if (jushRoot) {
-		var script = document.createElement('script');
-		script.src = jushRoot + 'jush.js';
-		script.onload = function () {
-			if (window.jush) { // IE runs in case of an error too
-				jush.create_links = ' target="_blank" rel="noreferrer"';
-				jush.urls.sql_sqlset = jush.urls.sql[0] = jush.urls.sqlset[0] = jush.urls.sqlstatus[0] = 'http://dev.mysql.com/doc/refman/' + version + '/en/$key';
-				var pgsql = 'http://www.postgresql.org/docs/' + version + '/static/';
-				jush.urls.pgsql_pgsqlset = jush.urls.pgsql[0] = pgsql + '$key';
-				jush.urls.pgsqlset[0] = pgsql + 'runtime-config-$key.html#GUC-$1';
-				jush.style(jushRoot + 'jush.css');
-				if (window.jushLinks) {
-					jush.custom_links = jushLinks;
+	if (window.jush) {
+		jush.create_links = ' target="_blank" rel="noreferrer"';
+		if (version) {
+			for (var key in jush.urls) {
+				var obj = jush.urls;
+				if (typeof obj[key] != 'string') {
+					obj = obj[key];
+					key = 0;
 				}
-				jush.highlight_tag('code', 0);
+				obj[key] = obj[key]
+					.replace(/\/doc\/mysql/, '/doc/refman/' + version) // MySQL
+					.replace(/\/docs\/current/, '/docs/' + version) // PostgreSQL
+				;
 			}
-		};
-		script.onreadystatechange = function () {
-			if (/^(loaded|complete)$/.test(script.readyState)) {
-				script.onload();
+		}
+		if (window.jushLinks) {
+			jush.custom_links = jushLinks;
+		}
+		jush.highlight_tag('code', 0);
+		var tags = document.getElementsByTagName('textarea');
+		for (var i = 0; i < tags.length; i++) {
+			if (/(^|\s)jush-/.test(tags[i].className)) {
+				var pre = jush.textarea(tags[i]);
+				if (pre) {
+					setupSubmitHighlightInput(pre);
+				}
 			}
-		};
-		document.body.appendChild(script);
+		}
 	}
 }
 
@@ -60,48 +60,105 @@ function typePassword(el, disable) {
 	}
 }
 
+/** Hide or show some login rows for selected driver
+* @param HTMLSelectElement
+*/
 function loginDriver(driver) {
-	var trs = driver.parentNode.parentNode.parentNode.rows;
-	for (var i=1; i < trs.length; i++) {
-		trs[i].className = (/sqlite/.test(driver.value) ? 'hidden' : '');
+	var trs = parentTag(driver, 'table').rows;
+	for (var i=1; i < trs.length - 1; i++) {
+		var disabled = /sqlite/.test(driver.value);
+		alterClass(trs[i], 'hidden', disabled);
+		trs[i].getElementsByTagName('input')[0].disabled = disabled;
 	}
 }
 
 
 
-/** Handle Tab and Esc in textarea
-* @param HTMLTextAreaElement
-* @param KeyboardEvent
-* @return boolean
+var dbCtrl;
+var dbPrevious = {};
+
+/** Check if database should be opened to a new window
+* @param MouseEvent
+* @param HTMLSelectElement
 */
-function textareaKeydown(target, event) {
-	if (!event.shiftKey && !event.altKey && !event.ctrlKey && !event.metaKey) {
-		if (event.keyCode == 9) { // 9 - Tab
-			// inspired by http://pallieter.org/Projects/insertTab/
-			if (target.setSelectionRange) {
-				var start = target.selectionStart;
-				var scrolled = target.scrollTop;
-				target.value = target.value.substr(0, start) + '\t' + target.value.substr(target.selectionEnd);
-				target.setSelectionRange(start + 1, start + 1);
-				target.scrollTop = scrolled;
-				return false; //! still loses focus in Opera, can be solved by handling onblur
-			} else if (target.createTextRange) {
-				document.selection.createRange().text = '\t';
-				return false;
+function dbMouseDown(event, el) {
+	dbCtrl = isCtrl(event);
+	if (dbPrevious[el.name] == undefined) {
+		dbPrevious[el.name] = el.value;
+	}
+}
+
+/** Load database after selecting it
+* @param HTMLSelectElement
+*/
+function dbChange(el) {
+	if (dbCtrl) {
+		el.form.target = '_blank';
+	}
+	el.form.submit();
+	el.form.target = '';
+	if (dbCtrl && dbPrevious[el.name] != undefined) {
+		el.value = dbPrevious[el.name];
+		dbPrevious[el.name] = undefined;
+	}
+}
+
+
+
+/** Check whether the query will be executed with index
+* @param HTMLFormElement
+*/
+function selectFieldChange(form) {
+	var ok = (function () {
+		var inputs = form.getElementsByTagName('input');
+		for (var i=0; i < inputs.length; i++) {
+			if (inputs[i].value && /^fulltext/.test(inputs[i].name)) {
+				return true;
 			}
 		}
-		if (event.keyCode == 27) { // 27 - Esc
-			var els = target.form.elements;
-			for (var i=1; i < els.length; i++) {
-				if (els[i-1] == target) {
-					els[i].focus();
-					break;
+		var ok = form.limit.value;
+		var selects = form.getElementsByTagName('select');
+		var group = false;
+		var columns = {};
+		for (var i=0; i < selects.length; i++) {
+			var select = selects[i];
+			var col = selectValue(select);
+			var match = /^(where.+)col\]/.exec(select.name);
+			if (match) {
+				var op = selectValue(form[match[1] + 'op]']);
+				var val = form[match[1] + 'val]'].value;
+				if (col in indexColumns && (!/LIKE|REGEXP/.test(op) || (op == 'LIKE' && val.charAt(0) != '%'))) {
+					return true;
+				} else if (col || val) {
+					ok = false;
 				}
 			}
-			return false;
+			if ((match = /^(columns.+)fun\]/.exec(select.name))) {
+				if (/^(avg|count|count distinct|group_concat|max|min|sum)$/.test(col)) {
+					group = true;
+				}
+				var val = selectValue(form[match[1] + 'col]']);
+				if (val) {
+					columns[col && col != 'count' ? '' : val] = 1;
+				}
+			}
+			if (col && /^order/.test(select.name)) {
+				if (!(col in indexColumns)) {
+					ok = false;
+				}
+				break;
+			}
 		}
-	}
-	return true;
+		if (group) {
+			for (var col in columns) {
+				if (!(col in indexColumns)) {
+					ok = false;
+				}
+			}
+		}
+		return ok;
+	})();
+	setHtml('noindex', (ok ? '' : '!'));
 }
 
 
@@ -166,21 +223,17 @@ function editingNameChange(field) {
 /** Add table row for next field
 * @param HTMLInputElement
 * @param boolean
-* @param boolean
 * @return boolean
 */
-function editingAddRow(button, allowed, focus) {
-	if (allowed && rowCount >= allowed) {
-		return false;
-	}
+function editingAddRow(button, focus) {
 	var match = /(\d+)(\.\d+)?/.exec(button.name);
 	var x = match[0] + (match[2] ? added.substr(match[2].length) : added) + '1';
-	var row = button.parentNode.parentNode;
-	var row2 = row.cloneNode(true);
+	var row = parentTag(button, 'tr');
+	var row2 = cloneNode(row);
 	var tags = row.getElementsByTagName('select');
 	var tags2 = row2.getElementsByTagName('select');
 	for (var i=0; i < tags.length; i++) {
-		tags2[i].name = tags[i].name.replace(/([0-9.]+)/, x);
+		tags2[i].name = tags[i].name.replace(/[0-9.]+/, x);
 		tags2[i].selectedIndex = tags[i].selectedIndex;
 	}
 	tags = row.getElementsByTagName('input');
@@ -202,11 +255,15 @@ function editingAddRow(button, allowed, focus) {
 	tags[0].onchange = function () {
 		editingNameChange(tags[0]);
 	};
+	tags[0].onkeyup = function () {
+	};
 	row.parentNode.insertBefore(row2, row.nextSibling);
 	if (focus) {
 		input.onchange = function () {
 			editingNameChange(input);
 		};
+		input.onkeyup = function () {
+		};
 		input.focus();
 	}
 	added += '0';
@@ -216,12 +273,13 @@ function editingAddRow(button, allowed, focus) {
 
 /** Remove table row for field
 * @param HTMLInputElement
+* @param string
 * @return boolean
 */
-function editingRemoveRow(button) {
-	var field = formField(button.form, button.name.replace(/drop_col(.+)/, 'fields$1[field]'));
+function editingRemoveRow(button, name) {
+	var field = formField(button.form, button.name.replace(/[^\[]+(.+)/, name));
 	field.parentNode.removeChild(field);
-	button.parentNode.parentNode.style.display = 'none';
+	parentTag(button, 'tr').style.display = 'none';
 	return true;
 }
 
@@ -235,25 +293,39 @@ function editingTypeChange(type) {
 	var text = selectValue(type);
 	for (var i=0; i < type.form.elements.length; i++) {
 		var el = type.form.elements[i];
-		if (el.name == name + '[length]' && !(
-			(/(char|binary)$/.test(lastType) && /(char|binary)$/.test(text))
-			|| (/(enum|set)$/.test(lastType) && /(enum|set)$/.test(text))
-		)) {
-			el.value = '';
+		if (el.name == name + '[length]') {
+			if (!(
+				(/(char|binary)$/.test(lastType) && /(char|binary)$/.test(text))
+				|| (/(enum|set)$/.test(lastType) && /(enum|set)$/.test(text))
+			)) {
+				el.value = '';
+			}
+			el.onchange.apply(el);
 		}
 		if (lastType == 'timestamp' && el.name == name + '[has_default]' && /timestamp/i.test(formField(type.form, name + '[default]').value)) {
 			el.checked = false;
 		}
 		if (el.name == name + '[collation]') {
-			el.className = (/(char|text|enum|set)$/.test(text) ? '' : 'hidden');
+			alterClass(el, 'hidden', !/(char|text|enum|set)$/.test(text));
 		}
 		if (el.name == name + '[unsigned]') {
-			el.className = (/(int|float|double|decimal)$/.test(text) ? '' : 'hidden');
+			alterClass(el, 'hidden', !/((^|[^o])int|float|double|decimal)$/.test(text));
+		}
+		if (el.name == name + '[on_update]') {
+			alterClass(el, 'hidden', !/timestamp|datetime/.test(text)); // MySQL supports datetime since 5.6.5
 		}
 		if (el.name == name + '[on_delete]') {
-			el.className = (/`/.test(text) ? '' : 'hidden');
+			alterClass(el, 'hidden', !/`/.test(text));
 		}
 	}
+	helpClose();
+}
+
+/** Mark length as required
+* @param HTMLInputElement
+*/
+function editingLengthChange(el) {
+	alterClass(el, 'required', !el.value.length && /var(char|binary)$/.test(selectValue(el.parentNode.previousSibling.firstChild)));
 }
 
 /** Edit enum or set
@@ -264,7 +336,7 @@ function editingLengthFocus(field) {
 	if (/(enum|set)$/.test(selectValue(td.previousSibling.firstChild))) {
 		var edit = document.getElementById('enum-edit');
 		var val = field.value;
-		edit.value = (/^'.+','.+'$/.test(val) ? val.substr(1, val.length - 2).replace(/','/g, "\n").replace(/''/g, "'") : val);
+		edit.value = (/^'.+'$/.test(val) ? val.substr(1, val.length - 2).replace(/','/g, "\n").replace(/''/g, "'") : val); //! doesn't handle 'a'',''b' correctly
 		td.appendChild(edit);
 		field.style.display = 'none';
 		edit.style.display = 'inline';
@@ -278,7 +350,7 @@ function editingLengthFocus(field) {
 function editingLengthBlur(edit) {
 	var field = edit.parentNode.firstChild;
 	var val = edit.value;
-	field.value = (/\n/.test(val) ? "'" + val.replace(/\n+$/, '').replace(/'/g, "''").replace(/\n/g, "','") + "'" : val);
+	field.value = (/^'[^\n]+'$/.test(val) ? val : "'" + val.replace(/\n+$/, '').replace(/'/g, "''").replace(/\n/g, "','") + "'");
 	field.style.display = 'inline';
 	edit.style.display = 'none';
 }
@@ -290,7 +362,16 @@ function editingLengthBlur(edit) {
 function columnShow(checked, column) {
 	var trs = document.getElementById('edit-fields').getElementsByTagName('tr');
 	for (var i=0; i < trs.length; i++) {
-		trs[i].getElementsByTagName('td')[column].className = (checked ? '' : 'hidden');
+		alterClass(trs[i].getElementsByTagName('td')[column], 'hidden', !checked);
+	}
+}
+
+/** Hide column with default values in narrow window
+*/
+function editingHideDefaults() {
+	if (innerWidth < document.documentElement.scrollWidth) {
+		document.getElementById('form')['defaults'].checked = false;
+		columnShow(false, 5);
 	}
 }
 
@@ -299,17 +380,18 @@ function columnShow(checked, column) {
 */
 function partitionByChange(el) {
 	var partitionTable = /RANGE|LIST/.test(selectValue(el));
-	el.form['partitions'].className = (partitionTable || !el.selectedIndex ? 'hidden' : '');
-	document.getElementById('partition-table').className = (partitionTable ? '' : 'hidden');
+	alterClass(el.form['partitions'], 'hidden', partitionTable || !el.selectedIndex);
+	alterClass(document.getElementById('partition-table'), 'hidden', !partitionTable);
+	helpClose();
 }
 
 /** Add next partition row
 * @param HTMLInputElement
 */
 function partitionNameChange(el) {
-	var row = el.parentNode.parentNode.cloneNode(true);
+	var row = cloneNode(parentTag(el, 'tr'));
 	row.firstChild.firstChild.value = '';
-	el.parentNode.parentNode.parentNode.appendChild(row);
+	parentTag(el, 'table').appendChild(row);
 	el.onchange = function () {};
 }
 
@@ -320,13 +402,13 @@ function partitionNameChange(el) {
 */
 function foreignAddRow(field) {
 	field.onchange = function () { };
-	var row = field.parentNode.parentNode.cloneNode(true);
+	var row = cloneNode(parentTag(field, 'tr'));
 	var selects = row.getElementsByTagName('select');
 	for (var i=0; i < selects.length; i++) {
 		selects[i].name = selects[i].name.replace(/\]/, '1$&');
 		selects[i].selectedIndex = 0;
 	}
-	field.parentNode.parentNode.parentNode.appendChild(row);
+	parentTag(field, 'table').appendChild(row);
 }
 
 
@@ -336,8 +418,7 @@ function foreignAddRow(field) {
 */
 function indexesAddRow(field) {
 	field.onchange = function () { };
-	var parent = field.parentNode.parentNode;
-	var row = parent.cloneNode(true);
+	var row = cloneNode(parentTag(field, 'tr'));
 	var selects = row.getElementsByTagName('select');
 	for (var i=0; i < selects.length; i++) {
 		selects[i].name = selects[i].name.replace(/indexes\[\d+/, '$&1');
@@ -348,7 +429,7 @@ function indexesAddRow(field) {
 		inputs[i].name = inputs[i].name.replace(/indexes\[\d+/, '$&1');
 		inputs[i].value = '';
 	}
-	parent.parentNode.appendChild(row);
+	parentTag(field, 'table').appendChild(row);
 }
 
 /** Change column in index
@@ -356,12 +437,16 @@ function indexesAddRow(field) {
 * @param string name prefix
 */
 function indexesChangeColumn(field, prefix) {
-	var columns = field.parentNode.parentNode.getElementsByTagName('select');
 	var names = [];
-	for (var i=0; i < columns.length; i++) {
-		var value = selectValue(columns[i]);
-		if (value) {
-			names.push(value);
+	for (var tag in { 'select': 1, 'input': 1 }) {
+		var columns = parentTag(field, 'td').getElementsByTagName(tag);
+		for (var i=0; i < columns.length; i++) {
+			if (/\[columns\]/.test(columns[i].name)) {
+				var value = selectValue(columns[i]);
+				if (value) {
+					names.push(value);
+				}
+			}
 		}
 	}
 	field.form[field.name.replace(/\].*/, '][name]')].value = prefix + names.join('_');
@@ -377,32 +462,59 @@ function indexesAddColumn(field, prefix) {
 	};
 	var select = field.form[field.name.replace(/\].*/, '][type]')];
 	if (!select.selectedIndex) {
-		select.selectedIndex = 3;
+		while (selectValue(select) != "INDEX" && select.selectedIndex < select.options.length) {
+			select.selectedIndex++;
+		}
 		select.onchange();
 	}
-	var column = field.parentNode.cloneNode(true);
-	select = column.getElementsByTagName('select')[0];
-	select.name = select.name.replace(/\]\[\d+/, '$&1');
-	select.selectedIndex = 0;
-	var input = column.getElementsByTagName('input')[0];
-	input.name = input.name.replace(/\]\[\d+/, '$&1');
-	input.value = '';
-	field.parentNode.parentNode.appendChild(column);
+	var column = cloneNode(field.parentNode);
+	var selects = column.getElementsByTagName('select');
+	for (var i = 0; i < selects.length; i++) {
+		select = selects[i];
+		select.name = select.name.replace(/\]\[\d+/, '$&1');
+		select.selectedIndex = 0;
+	}
+	var inputs = column.getElementsByTagName('input');
+	for (var i = 0; i < inputs.length; i++) {
+		var input = inputs[i];
+		input.name = input.name.replace(/\]\[\d+/, '$&1');
+		if (input.type != 'checkbox') {
+			input.value = '';
+		}
+	}
+	parentTag(field, 'td').appendChild(column);
 	field.onchange();
 }
 
 
 
-var that, x, y, em, tablePos;
+/** Handle changing trigger time or event
+* @param RegExp
+* @param string
+* @param HTMLFormElement
+*/
+function triggerChange(tableRe, table, form) {
+	var formEvent = selectValue(form['Event']);
+	if (tableRe.test(form['Trigger'].value)) {
+		form['Trigger'].value = table + '_' + (selectValue(form['Timing']).charAt(0) + formEvent.charAt(0)).toLowerCase();
+	}
+	alterClass(form['Of'], 'hidden', formEvent != 'UPDATE OF');
+}
+
+
+
+var that, x, y; // em and tablePos defined in schema.inc.php
 
 /** Get mouse position
 * @param HTMLElement
 * @param MouseEvent
 */
 function schemaMousedown(el, event) {
-	that = el;
-	x = event.clientX - el.offsetLeft;
-	y = event.clientY - el.offsetTop;
+	if ((event.which ? event.which : event.button) == 1) {
+		that = el;
+		x = event.clientX - el.offsetLeft;
+		y = event.clientY - el.offsetTop;
+	}
 }
 
 /** Move object
@@ -417,10 +529,9 @@ function schemaMousemove(ev) {
 		var lineSet = { };
 		for (var i=0; i < divs.length; i++) {
 			if (divs[i].className == 'references') {
-				var div2 = document.getElementById((divs[i].id.substr(0, 4) == 'refs' ? 'refd' : 'refs') + divs[i].id.substr(4));
+				var div2 = document.getElementById((/^refs/.test(divs[i].id) ? 'refd' : 'refs') + divs[i].id.substr(4));
 				var ref = (tablePos[divs[i].title] ? tablePos[divs[i].title] : [ div2.parentNode.offsetTop / em, 0 ]);
 				var left1 = -1;
-				var isTop = true;
 				var id = divs[i].id.replace(/^ref.(.+)-.+/, '$1');
 				if (divs[i].parentNode != div2.parentNode) {
 					left1 = Math.min(0, ref[1] - left) - 1;
@@ -429,19 +540,17 @@ function schemaMousemove(ev) {
 					var left2 = Math.min(0, left - ref[1]) - 1;
 					div2.style.left = left2 + 'em';
 					div2.getElementsByTagName('div')[0].style.width = -left2 + 'em';
-					isTop = (div2.offsetTop + ref[0] * em > divs[i].offsetTop + top * em);
 				}
 				if (!lineSet[id]) {
-					var line = document.getElementById(divs[i].id.replace(/^....(.+)-\d+$/, 'refl$1'));
-					var shift = ev.clientY - y - that.offsetTop;
-					line.style.left = (left + left1) + 'em';
-					if (isTop) {
-						line.style.top = (line.offsetTop + shift) / em + 'em';
-					}
+					var line = document.getElementById(divs[i].id.replace(/^....(.+)-.+$/, 'refl$1'));
+					var top1 = top + divs[i].offsetTop / em;
+					var top2 = top + div2.offsetTop / em;
 					if (divs[i].parentNode != div2.parentNode) {
-						line = line.getElementsByTagName('div')[0];
-						line.style.height = (line.offsetHeight + (isTop ? -1 : 1) * shift) / em + 'em';
+						top2 += ref[0] - top;
+						line.getElementsByTagName('div')[0].style.height = Math.abs(top1 - top2) + 'em';
 					}
+					line.style.left = (left + left1) + 'em';
+					line.style.top = Math.min(top1, top2) + 'em';
 					lineSet[id] = true;
 				}
 			}
@@ -470,3 +579,50 @@ function schemaMouseup(ev, db) {
 		cookie('adminer_schema-' + db + '=' + s, 30); //! special chars in db
 	}
 }
+
+
+
+var helpOpen, helpIgnore; // when mouse outs <option> then it mouse overs border of <select> - ignore it
+
+/** Display help
+* @param HTMLElement
+* @param MouseEvent
+* @param string
+* @param bool display on left side (otherwise on top)
+*/
+function helpMouseover(el, event, text, side) {
+	var target = getTarget(event);
+	if (!text) {
+		helpClose();
+	} else if (window.jush && (!helpIgnore || el != target)) {
+		helpOpen = 1;
+		var help = document.getElementById('help');
+		help.innerHTML = text;
+		jush.highlight_tag([ help ]);
+		alterClass(help, 'hidden');
+		var rect = target.getBoundingClientRect();
+		var body = document.documentElement;
+		help.style.top = (body.scrollTop + rect.top - (side ? (help.offsetHeight - target.offsetHeight) / 2 : help.offsetHeight)) + 'px';
+		help.style.left = (body.scrollLeft + rect.left - (side ? help.offsetWidth : (help.offsetWidth - target.offsetWidth) / 2)) + 'px';
+	}
+}
+
+/** Close help after timeout
+* @param HTMLElement
+* @param MouseEvent
+*/
+function helpMouseout(el, event) {
+	helpOpen = 0;
+	helpIgnore = (el != getTarget(event));
+	setTimeout(function () {
+		if (!helpOpen) {
+			helpClose();
+		}
+	}, 200);
+}
+
+/** Close help
+*/
+function helpClose() {
+	alterClass(document.getElementById('help'), 'hidden', true);
+}
