@@ -1,18 +1,22 @@
 // Editor specific functions
 
-function bodyLoad(version) {
-	if (history.state !== undefined) {
-		onpopstate(history);
-	}
+function selectFieldChange() {
+}
+
+var helpOpen;
+
+function helpMouseover() {
+}
+
+function helpMouseout() {
 }
 
 function whisperClick(event, field) {
-	var el = event.target || event.srcElement;
-	if (/^a$/i.test(el.tagName) && !(event.button || event.ctrlKey || event.shiftKey || event.altKey || event.metaKey)) {
+	var el = getTarget(event);
+	if (isTag(el, 'a') && !(event.button || event.shiftKey || event.altKey || isCtrl(event))) {
 		field.value = el.firstChild.data;
 		field.previousSibling.value = decodeURIComponent(el.href.replace(/.*=/, ''));
 		field.nextSibling.style.display = 'none';
-		eventStop(event);
 		return false;
 	}
 }
