@@ -1,6 +1,7 @@
 <?php
 
 /** Dump to ZIP format
+* @link http://www.adminer.org/plugins/#use
 * @uses ZipArchive, tempnam("")
 * @author Jakub Vrana, http://www.vrana.cz/
 * @license http://www.apache.org/licenses/LICENSE-2.0 Apache License, Version 2.0
@@ -8,23 +9,23 @@
 */
 class AdminerDumpZip {
 	/** @access protected */
-	var $filename;
-	
+	var $filename, $data;
+
 	function dumpOutput() {
 		if (!class_exists('ZipArchive')) {
 			return array();
 		}
 		return array('zip' => 'ZIP');
 	}
-	
+
 	function _zip($string, $state) {
-		static $data = "";
-		$data .= $string;
+		// ZIP can be created without temporary file by gzcompress - see PEAR File_Archive
+		$this->data .= $string;
 		if ($state & PHP_OUTPUT_HANDLER_END) {
 			$zip = new ZipArchive;
 			$zipFile = tempnam("", "zip");
 			$zip->open($zipFile, ZipArchive::OVERWRITE); // php://output is not supported
-			$zip->addFromString($this->filename, $data);
+			$zip->addFromString($this->filename, $this->data);
 			$zip->close();
 			$return = file_get_contents($zipFile);
 			unlink($zipFile);
@@ -32,13 +33,13 @@ class AdminerDumpZip {
 		}
 		return "";
 	}
-	
+
 	function dumpHeaders($identifier, $multi_table = false) {
-		$this->filename = "$identifier." . ($multi_table && ereg("[ct]sv", $_POST["format"]) ? "tar" : $_POST["format"]);
 		if ($_POST["output"] == "zip") {
+			$this->filename = "$identifier." . ($multi_table && preg_match("~[ct]sv~", $_POST["format"]) ? "tar" : $_POST["format"]);
 			header("Content-Type: application/zip");
+			ob_start(array($this, '_zip'));
 		}
-		ob_start(array($this, '_zip'));
 	}
 
 }
