@@ -1,6 +1,7 @@
 <?php
 
 /** Use <select><option> for enum edit instead of <input type="radio">
+* @link http://www.adminer.org/plugins/#use
 * @author Jakub Vrana, http://www.vrana.cz/
 * @license http://www.apache.org/licenses/LICENSE-2.0 Apache License, Version 2.0
 * @license http://www.gnu.org/licenses/gpl-2.0.html GNU General Public License, version 2 (one or other)
@@ -16,7 +17,7 @@ class AdminerEnumOption {
 			}
 			if ($field["null"]) {
 				$options[""][""] = "NULL";
-				if (!isset($value) && !isset($_GET["select"])) {
+				if ($value === null && !isset($_GET["select"])) {
 					$selected = "";
 				}
 			}
